from typing import List, Optional

from .formats import get_format, get_format_set_suitable_function
from .gmx_spells import gmx_get_first_frame

# Set the functions to performe first frame gettering

# Get the first frame from a trajectory
# Return the single frame filename
first_frame_getter_functions = [ gmx_get_first_frame ]
def get_first_frame ( input_trajectory_filename : str, accepted_output_formats : Optional[List[str]] = None ) -> str:
    # Get the input trajecotry format
    input_format = get_format(input_trajectory_filename)
    format_set = {
        'input_structure_filename': None,
        'input_trajectory_filename': [input_format],
        'output_frame_filename': accepted_output_formats
    }
    # Get a suitable function to obtain the unique frame
    suitables = get_format_set_suitable_function(
        available_functions=first_frame_getter_functions,
        available_request_format_sets=[format_set],
    )
    if not suitables:
        raise SystemExit('There is no first frame getter function which supports the requested formats')
    suitable_function, formats = suitables
    # The output format will be the first common format between the available formats and the function formats
    output_format = formats['outputs']['output_frame_filename'][0]
    # Set the output filename
    output_single_frame_filename = '.single_frame.' + output_format
    suitable_function(input_trajectory_filename, output_single_frame_filename)
    return output_single_frame_filename



import pytraj as pyt

# Get the trajectory frames number using pytraj
def get_frames_count (
    input_topology_filename : str,
    input_trajectory_filename : str) -> int:
    
    # Load the trajectory from pytraj
    pyt_trajectory = pyt.iterload(
        input_trajectory_filename,
        input_topology_filename)

    # Return the frames number
    return pyt_trajectory.n_frames


from argparse import ArgumentParser, RawTextHelpFormatter

from .conversions import convert
from .vmd_spells import chainer

# Define console call for mdtoolbelt
parser = ArgumentParser(description="Call a tool from mdtoolbelt", formatter_class=RawTextHelpFormatter)
subparsers = parser.add_subparsers(help='Name of the tool to be used', dest="tool")

# The convert command
convert_parser = subparsers.add_parser("convert")
convert_parser.add_argument(
    "-is", "--input_structure",
    help="Path to input structure file")
convert_parser.add_argument(
    "-os", "--output_structure",
    help="Path to output structure file")
convert_parser.add_argument(
    "-it", "--input_trajectory",
    help="Path to input trajectory file")
convert_parser.add_argument(
    "-ot", "--output_trajectory",
    help="Path to output trajectory file")

# The chainer command
chainer_parser = subparsers.add_parser("chainer")
chainer_parser.add_argument(
    "-is", "--input_structure", required=True,
    help="Path to input structure file (pdb)")
chainer_parser.add_argument(
    "-sel", "--atom_selection",
    help="Atoms to be chained")
chainer_parser.add_argument(
    "-chn", "--chain_letter",
    help="The chain letter to be set in selected atoms")
chainer_parser.add_argument(
    "-os", "--output_structure",
    help="Path to output structure file (pdb)")



args = parser.parse_args()

def call():

    # If no command is passed print help
    tool = args.tool
    if not tool:
        parser.print_help()
        return

    # In case the convert tool was called
    if tool == 'convert':
        # If no input arguments are passed print help
        if args.input_structure == None and args.input_trajectory == None:
            convert_parser.print_help()
            return
        # Run the convert command
        convert(
            input_structure_filename=args.input_structure,
            output_structure_filename=args.output_structure,
            input_trajectory_filename=args.input_trajectory,
            output_trajectory_filename=args.output_trajectory,
        )

    # In case the chainer tool was called
    if tool == 'chainer':
        chainer(
            input_pdb_filename=args.input_structure,
            atom_selection=args.atom_selection,
            chain_letter=args.chain_letter,
            output_pdb_filename=args.output_structure
        )

    # Tool will always match one of the previous defined options
    # Otherwise argparse returns error itself


import os
from subprocess import run, PIPE
import mdtraj as md

# Multiple files may be selected with bash syntax (e.g. *.dcd)
# Tested supported input formats are .dcd
# Tested supported output formats are .xtc
def merge_and_convert_traj (
    input_filenames : list,
    output_filename : str
    ):

    # Run MDtraj
    logs = run([
        "mdconvert",
        "-o",
        output_filename,
        *input_filenames,
    ], stderr=PIPE).stderr.decode()
    # If output has not been generated then warn the user
    if not os.path.exists(output_filename):
        print(logs)
        raise SystemExit('Something went wrong with MDTraj')

def convert_traj (input_trajectory_filename : str, output_trajectory_filename : str):
    merge_and_convert_traj([input_trajectory_filename], output_trajectory_filename)
convert_traj.format_sets = [
    {
        'inputs': {
            'input_trajectory_filename': {'dcd', 'xtc', 'trr', 'nc', 'h5', 'binpos'}
        },
        'outputs': {
            'output_trajectory_filename': {'dcd', 'xtc', 'trr', 'nc', 'h5', 'binpos'}
        }
    },
]

# Get specific frames from a trajectory
def get_trajectory_subset (
    input_structure_filename : str,
    input_trajectory_filename : str,
    output_trajectory_filename : str,
    start : int = 0,
    end : int = 0,
    step : int = 1
):
    # In case no end is passed return the only the start frame
    if not end:
        end = start + 1

    # Load the trajectory frame by frame and get only the desired frames
    trajectory = md.iterload(input_trajectory_filename, top=input_structure_filename, chunk=1)
    # Get the first chunk
    reduced_trajectory = None
    for i, chunk in enumerate(trajectory):
        if i == start:
            reduced_trajectory = chunk
            break
    # Get further chunks
    relative_end = end - start
    for i, chunk in enumerate(trajectory, 1): # Start the count at 1
        if i == relative_end:
            break
        if i % step == 0:
            reduced_trajectory = md.join([reduced_trajectory, chunk], check_topology=False)

    # Write reduced trajectory to output file
    reduced_trajectory.save(output_trajectory_filename)
get_trajectory_subset.format_sets = [
    {
        'inputs': {
            'input_structure_filename': {'pdb', 'h5'},
            'input_trajectory_filename': {'dcd', 'xtc', 'trr', 'nc', 'binpos'}
        },
        'outputs': {
            'output_trajectory_filename': {'dcd', 'xtc', 'trr', 'nc', 'binpos'}
        }
    },
    {
        'inputs': {
            'input_structure_filename': None,
            'input_trajectory_filename': {'pdb', 'h5'}
        },
        'outputs': {
            'output_trajectory_filename': {'pdb', 'h5'}
        }
    }
]